#ifndef __ASM_GENERIC_MUTEX_H
#define __ASM_GENERIC_MUTEX_H
/*
 * Pull in the generic implementation for the mutex fastpath,
 * which is a reasonable default on many architectures.
 */

#include <asm-generic/mutex-dec.h>
#endif /* __ASM_GENERIC_MUTEX_H */
